/* Colors for badges */

.badge-red {
  background-color: #d54625;
}

.badge-green {
  background-color: #35bd4a;
}

.badge-blue {
  background-color: #357ebd;
}

.table-sortable > thead > tr > th {
  cursor: pointer;
}
.table-sortable > thead > tr > th[data-sorting="disabled"] {
  cursor: default;
}
.table-sortable > thead > tr > th[data-sorting="disabled"]:hover,
.table-sortable > thead > tr > th[data-sorting="disabled"]:active {
  background: inherit;
}
.table-sortable > thead > tr > th:hover,
.table-sortable > thead > tr > th:active {
  background: #F5F5F5;
}
.table-sortable > tbody > tr > td mark,
.table-sortable > tbody > tr > td .mark {
  padding-left: 0;
  padding-right: 0;
}
